%define gitrev f80c2cf
%define gitdate 20160301

# How to prepare the source archive from the git tree:
# $ hash=`git log -1 --format='%h'`
# $ date=`git log -1 --format='%cd' --date=short | tr -d -`
# $ git archive --prefix kpatch-${date}-git${hash}/ ${hash} \
#       | xz -7e > kpatch-${date}-git${hash}.tar.xz


Name:		kpatch
Version:	0.3.3
Release:	18.%{gitdate}%{?dist}
Summary:	Dynamic kernel patch manager

Group:		System Environment/Kernel
License:	GPLv2
URL:		https://github.com/dynup/kpatch
Source0:	%{name}-%{gitdate}-git%{gitrev}.tar.xz
Source1:	kpatch-logrotate.conf

# The patches from upstream, the numbers should start from 0.
# Submitted upstream: https://github.com/dynup/kpatch/pull/588
Patch0:		0001-contrib-service-allow-to-turn-the-service-off.patch

# Virtuozzo-specific patches.
Patch501:	vz001-kpatch-build-Support-Virtuozzo-Linux-as-well.patch
Patch502:	vz002-kpatch-Output-more-info-about-the-patch.patch
Patch503:	vz003-service-Use-kpatch-with-retries.patch
Patch504:	vz004-kpatch-with-retries-output-errors-to-the-system-log.patch
Patch505:	vz005-patch-hook-Do-not-create-sysfs-files-for-functions.patch
Patch506:	vz006-Revert-kpatch-deprecate-the-replace-command.patch
Patch507:	vz007-Kpatch-prerelease-version.patch
Patch508:	vz008-fix-kpatch-info-if-a-patch-module-fails-to-load.patch
Patch509:	vz009-Do-not-mention-initrd-in-the-help-messages.patch
Patch510:	vz010-Added-logging-to-var-log-kpatch.patch
Patch511:	vz011-Implement-kpatch-uninstall-all.patch
Patch512:	vz012-Revisited-kpatch-info-to-handle-versionized-info.txt.patch
Patch513:	readykernel-info.patch

ExclusiveArch:	x86_64

BuildRequires:	elfutils-devel

Requires:	bash
Requires:	kmod
Requires:	/usr/bin/logger
Requires:	logrotate


%description
Kpatch is a Linux dynamic kernel patching tool which allows you to patch
a running kernel without rebooting or restarting any processes. It enables
sysadmins to apply critical security patches to the kernel immediately,
without having to wait for long-running tasks to complete, users to
log off, or for scheduled reboot windows. It gives more control over
up-time without sacrificing security or stability.


%package build
Summary:	The tools to build Kpatch-based kernel patches
Group:		Development/Tools
License:	GPLv2

Requires:	gcc >= 4.8
Requires:	elfutils
Requires:	ccache
Requires:	rpmdevtools
Requires:	yum-utils
Requires:	pesign
Requires:	openssl
Requires:	numactl-devel
Requires:	ncurses-devel
Requires:	bash
Requires:	findutils
Requires:	perl
Requires:	net-tools
Requires:	bc
Requires:	xmlto
Requires:	asciidoc
Requires:	hmaccalc
Requires:	python-devel
Requires:	newt-devel
Requires:	perl(ExtUtils::Embed)

# When building Kpatch-based rather than Livepatch-based binary patches,
# Module.symvers of the Kpatch core module is needed. However, it is not
# required for the users to use our kpatch-kmod package. They might want to
# build the core module themselves, or, alternatively, use Livepatch.
# So, kpatch-kmod is not listed as a requirement here.


%description build
This package contains the tools needed to prepare binary kernel patches
for Kpatch to use.


%prep
%setup -qn %{name}-%{gitdate}-git%{gitrev}
%patch0 -p1
%patch501 -p1
%patch502 -p1
%patch503 -p1
%patch504 -p1
%patch505 -p1
%patch506 -p1
%patch507 -p1
%patch508 -p1
%patch509 -p1
%patch510 -p1
%patch511 -p1
%patch512 -p1
%patch513 -p1


%build
make -C man
make -C kpatch-build %{_smp_mflags}


%install
rm -rf %{buildroot}
make install PREFIX=%{_usr} DESTDIR=%{buildroot} -C kpatch
make install PREFIX=%{_usr} DESTDIR=%{buildroot} -C kpatch-build
make install PREFIX=%{_usr} DESTDIR=%{buildroot} -C man
make install PREFIX=%{_usr} DESTDIR=%{buildroot} -C contrib
install -d %{buildroot}%{_datadir}/%{name}/patch
install -m 644 kmod/patch/* %{buildroot}%{_datadir}/%{name}/patch/
install -d %{buildroot}%{_sysconfdir}/logrotate.d
install -m 644 %{SOURCE1} %{buildroot}%{_sysconfdir}/logrotate.d/%{name}

# A convenience symlink to allow using "readykernel info", etc.
pushd %{buildroot}%{_usr}/sbin
ln -s kpatch readykernel
popd


%files
%defattr(-,root,root,-)
%doc COPYING README.md
%{_sbindir}/kpatch
%{_sbindir}/kpatch-with-retries
%{_sbindir}/readykernel
%{_mandir}/man1/kpatch.1*
%{_usr}/lib/systemd/system/kpatch.service
%{_sysconfdir}/logrotate.d/%{name}


%files build
%defattr(-,root,root,-)
%{_bindir}/kpatch-build
%{_libexecdir}/%{name}/kpatch-gcc
%{_libexecdir}/%{name}/create-diff-object
%{_mandir}/man1/kpatch-build.1*
%{_datadir}/%{name}/patch/*


%changelog
* Mon Jun 6 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.3-18.20160301
- added a convenience symlink to allow using "readykernel info", etc.

* Tue May 31 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.3-17.20160301
- make sure "kpatch info" looks for data in appropriate places
- drop support for unversioned info.txt files

* Mon May 23 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.3-15.20160301
- fixed the units in the logrotate configuration file (PSBM-47253)

* Tue May 10 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.3-11.20160301
- added logging to /var/log/kpatch.log with logrotate integration
- minor fixes

* Fri May 6 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.3-10.20160301
- simplify the 'release' expression, similar to kpatch-kmod
- several fixes and improvements in "kpatch info"
- restore "kpatch replace"
- explicitly specify that we are using a pre-release version

* Mon Mar 14 2016 Evgenii Shatokhin <eshatokhin@virtuozzo.com> 0.3.2-1.20160301.1
- update to revision f80c2cf from the upstream git (v.0.3.2+)

* Tue Nov 24 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.3.1-0.20151118.1
- update to revision b60d3ac from the upstream git (v.0.3.1+)
- move kpatch-with-retries to a patch (helps rebasing)

* Wed Nov 11 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.2.2-0.20151104.1
- update to revision eb54876 from the upstream git

* Mon Oct 26 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.2.2-0.20151021.1
- update to revision ff40e54 from the upstream git
- use kpatch-with-retries to make patch application more reliable

* Mon Oct 19 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.2.2-0.20151016.1
- update to revision d444caa from the upstream git
- more fixes needed for mock-based package builds

* Wed Oct 14 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.2.2-0.20151001.3
- adjusted the patches and the requirements for RPM build with mock
- cosmetic improvements

* Thu Oct 8 2015 Evgenii Shatokhin <eshatokhin@odin.com> 0.2.2-0.20151001.1
- update to revision f049dcf from the upstream git
- add support for Virtuozzo 7
- build kpatch-build package as well

* Tue Sep 16 2014 Seth Jennings <sjenning@redhat.com> 0.1.10-4
- fix dracut dependencies (rhbz#1170369)

* Tue Sep 16 2014 Seth Jennings <sjenning@redhat.com> 0.1.10-3
- support re-enabling forced modules (rhbz#1140268)

* Thu Sep 11 2014 Seth Jennings <sjenning@redhat.com> 0.1.10-2
- support modprobe format names (rhbz#1133045)

* Thu Jul 31 2014 Josh Poimboeuf <jpoimboe@redhat.com> 0.1.10-1
- update to kpatch 0.1.10

* Wed Jul 23 2014 Josh Poimboeuf <jpoimboe@redhat.com> 0.1.9-1
- update to kpatch 0.1.9

* Tue Jul 15 2014 Josh Poimboeuf <jpoimboe@redhat.com> 0.1.8-1
- update to kpatch 0.1.8

* Wed May 21 2014 Josh Poimboeuf <jpoimboe@redhat.com> 0.1.2-1
- update to kpatch 0.1.2

* Mon May 19 2014 Josh Poimboeuf <jpoimboe@redhat.com> 0.1.1-2
- fix initramfs core module path

* Mon May 19 2014 Josh Poimboeuf <jpoimboe@redhat.com> 0.1.1-1
- rebase to kpatch 0.1.1

* Fri May 9 2014 Josh Poimboeuf <jpoimboe@redhat.com> 0.1.0-2
- modprobe core module

* Tue May 6 2014 Josh Poimboeuf <jpoimboe@redhat.com> 0.1.0-1
- Initial kpatch release 0.1.0

* Thu Jan 30 2014 Josh Poimboeuf <jpoimboe@redhat.com> 0.0-1
- Initial build
